@@ -41,14 +41,30 @@
 
 typedef struct {
         prelude_list_t list;
-        pcre *regex_compiled;
-        pcre_extra *regex_extra;
-        int options;
+
+        pcre *source_regex;
+
+        char *regex;
+        pcre *regex_regex;
+        pcre_extra *regex_regex_extra;
+
+        int line;
+        char *plugin;
+} regex_table_item_t;
+
+
+typedef struct {
+        prelude_list_t list;
+        regex_table_item_t *rt;
         prelude_plugin_instance_t *plugin;
 } regex_entry_t;
 
 
 
+static PRELUDE_LIST(regex_conf_list);
+
+
+
 static char *trim(char *str)
 {
         char *rptr, *wptr;        
@@ -87,10 +103,7 @@ static regex_entry_t *regex_entry_new(regex_list_t *list)
                 return NULL;
         }
 
-        new->regex_compiled = NULL;
-        new->regex_extra = NULL;
-        new->options = 0;
-
+        new->rt = NULL;
         prelude_list_add_tail(list, &new->list);
 
         return new;
@@ -102,47 +115,27 @@ static regex_entry_t *regex_entry_new(regex_list_t *list)
 static void regex_entry_delete(regex_entry_t *entry)
 {
         prelude_list_del(&entry->list);
-
-        if ( entry->regex_compiled )
-                pcre_free(entry->regex_compiled);
-
-        if ( entry->regex_extra )
-                pcre_free(entry->regex_extra);
-        
         free(entry);
 }
 
 
 
 
-static int regex_create_entry(regex_list_t *list, int line, const char *source,
-                              const char *pname, const char *regex, const char *options) 
+static int regex_create_entry(regex_list_t *list, regex_table_item_t *rt, const char *source) 
 {
-        int erroffset;
-        pcre *compiled;
-        const char *errptr;
         regex_entry_t *entry;
         prelude_plugin_generic_t *plugin;
-
-        compiled = pcre_compile(regex, 0, &errptr, &erroffset, NULL);
-        if ( ! compiled ) {
-                prelude_log(PRELUDE_LOG_WARN, "%s:%d : unable to compile: %s.\n", REGEX_CONF, line, errptr);
-                return -1;
-        }
         
         entry = regex_entry_new(list);
-        if ( ! entry ) {
-                pcre_free(compiled);
+        if ( ! entry )
                 return -1;
-        }
 
-        entry->regex_compiled = compiled;
-        entry->regex_extra = pcre_study(entry->regex_compiled, 0, &errptr);
-
-        entry->plugin = log_plugin_register(pname);
+        entry->rt = rt;
+        
+        entry->plugin = log_plugin_register(rt->plugin);
         if ( ! entry->plugin ) {
                 regex_entry_delete(entry);
-                prelude_log(PRELUDE_LOG_WARN, "%s:%d : couldn't find plugin: %s.\n", REGEX_CONF, line, pname);
+                prelude_log(PRELUDE_LOG_WARN, "%s:%d : couldn't find plugin: %s.\n", REGEX_CONF, rt->line, rt->plugin);
                 return -1;
         }
 
@@ -150,11 +143,7 @@ static int regex_create_entry(regex_list_t *list, int line, const char *source,
         prelude_log(PRELUDE_LOG_INFO, "- Monitoring %s through %s[%s]\n",
                     source, plugin->name, prelude_plugin_instance_get_name(entry->plugin));
         
-        /*
-         * TBD: take care of options field
-         */
-        
-        prelude_log(PRELUDE_LOG_DEBUG, "[REGEX] rule found: plugin: %s - pattern: %s - options: %s\n", pname, regex, options);
+        prelude_log(PRELUDE_LOG_DEBUG, "[REGEX] rule found: plugin: %s - pattern: %s\n", rt->plugin, rt->regex);
         
         return 0;
 }
@@ -162,15 +151,109 @@ static int regex_create_entry(regex_list_t *list, int line, const char *source,
 
 
 
-
-regex_list_t *regex_init(const char *source)
+static int get_regex_table(void)
 {
         FILE *fd;
         size_t len;
         char buf[1024];
-        int line = 1, ret;
+        const char *errptr;
+        int line = 0, erroff;
+        regex_table_item_t *rt;
+        pcre_extra *regex_regex_extra = NULL;
+        char *regex, *options, *source, *plugin;
+        pcre *regex_regex = NULL, *source_regex = NULL;
+        
+        fd = fopen(REGEX_CONF, "r");
+        if ( ! fd ) {
+                prelude_log(PRELUDE_LOG_ERR, "couldn't open config file %s.\n", REGEX_CONF);
+                return -1;
+        }
+
+        while ( fgets(buf, sizeof(buf), fd) ) {
+                trim(buf);
+                line++;
+
+                if ( buf[0] == '#' || buf[0] == '\0' )
+                        /*
+                         * ignore comments and empty lines
+                         */
+                        continue;
+
+                source = strtok(buf, " \t");
+                if ( ! source )
+                        continue;
+                
+                plugin = strtok(NULL, " \t");
+                if ( ! plugin )
+                        continue;
+                
+                options = strtok(NULL, " \t");
+                if ( ! options )
+                        continue;
+                
+                regex = strtok(NULL, "");
+                if ( ! regex )
+                        continue;
+
+                for ( len = strlen(regex); len && regex[len - 1] == ' '; len--)
+                        regex[len - 1] = 0;          
+
+                if ( strcmp(source, "*") != 0 ) {
+                        source_regex = pcre_compile(source, 0, &errptr, &erroff, NULL);
+                        if ( ! source_regex ) {
+                                prelude_log(PRELUDE_LOG_WARN, "%s:%d: Unable to compile source regexp: %s.\n", REGEX_CONF, line, errptr);
+                                continue;
+                        }
+                }
+
+                if ( strcmp(regex, "*") != 0 ) {
+                        regex_regex = pcre_compile(regex, 0, &errptr, &erroff, NULL);
+                        if ( ! regex_regex ) {
+                                if ( source_regex )
+                                        pcre_free(source_regex);
+                                
+                                prelude_log(PRELUDE_LOG_WARN, "%s:%d: Unable to compile regexp: %s.\n", REGEX_CONF, line, errptr);
+                                continue;
+                        }
+                        
+                        regex_regex_extra = pcre_study(regex_regex, 0, &errptr);
+                }
+              
+                rt = malloc(sizeof(*rt));
+                if ( ! rt ) {
+                        fclose(fd);
+                        return -1;
+                }
+                
+                rt->line = line; 
+                rt->source_regex = source_regex;
+                
+                rt->regex = strdup(regex);
+                rt->regex_regex = regex_regex;
+                rt->regex_regex_extra = regex_regex_extra;
+                
+                rt->plugin = strdup(plugin);
+
+                prelude_list_add_tail(&regex_conf_list, &rt->list);                
+        }
+        
+        fclose(fd);
+        return 0;
+}
+
+
+regex_list_t *regex_init(const char *source)
+{
+        int ret;
         regex_list_t *conf;
-        char *name, *regex, *options, *file;
+        prelude_list_t *tmp;
+        regex_table_item_t *rt;
+
+        if ( prelude_list_is_empty(&regex_conf_list) ) {
+                ret = get_regex_table(); 
+                if ( ret < 0 )
+                        return NULL;
+        }
 
         conf = malloc(sizeof(*conf));
         if ( ! conf ) {
@@ -178,66 +261,24 @@ regex_list_t *regex_init(const char *source)
                 return NULL;
         }
         
-        prelude_list_init(conf);
-
-        fd = fopen(REGEX_CONF, "r");
-        if ( ! fd ) {
-                prelude_log(PRELUDE_LOG_ERR, "couldn't open config file %s.\n", REGEX_CONF);
-                return NULL;
-        }
-
-        while ( fgets(buf, sizeof(buf), fd) ) {
-                trim(buf);
+        prelude_list_init(conf);                
                 
-                if ( buf[0] == '#' || buf[0] == '\0' )
-                        /*
-                         * ignore comments and empty lines
-                         */
-                        continue;
-
-                file = strtok(buf, " \t");
-                if ( ! file ) {
-                        line++;
-                        continue;
+        prelude_list_for_each(&regex_conf_list, tmp) {
+                rt = prelude_list_entry(tmp, regex_table_item_t, list);
+                
+                if ( rt->source_regex != NULL ) {
+                        ret = pcre_exec(rt->source_regex, NULL, source, strlen(source), 0, 0, NULL, 0);
+                        if ( ret < 0 )
+                                continue;
                 }
-                
-                if ( strcmp(file, source) != 0 && strcmp(file, "*") != 0 ) {
-                        line++;
-                        continue;
-                }
-                
-                name = strtok(NULL, " \t");
-                if ( ! name ) {
-                        line++;
-                        continue;
-                }
-                
-                options = strtok(NULL, " \t");
-                if ( ! options ) {
-                        line++;
-                        continue;
-                }
-                
-                regex = strtok(NULL, "");
-                if ( ! regex ) {
-                        line++;
-                        continue;
-                }
-
-                for ( len = strlen(regex); len && regex[len - 1] == ' '; len--)
-                        regex[len - 1] = 0;                
-                
-                ret = regex_create_entry(conf, line, source, name, regex, options);
+                   
+                ret = regex_create_entry(conf, rt, source);
                 if ( ret < 0 )
                         continue;
-                
-                line++;
         }
         
-        fclose(fd);
-
         if ( prelude_list_is_empty(conf) ) {
-                prelude_log(PRELUDE_LOG_WARN, "No plugin configured to receive %s data.\n", source);
+                prelude_log(PRELUDE_LOG_WARN, "* WARNING: No plugin configured to receive data from source '%s'.\n", source);
                 free(conf);
                 return NULL;
         }
@@ -268,19 +309,20 @@ int regex_exec(regex_list_t *list,
                void (*cb)(void *plugin, void *data), void *data,
                const char *str, size_t len)
 {
+        int ret;
         prelude_list_t *tmp;
         regex_entry_t *entry;
-        int count, ovector[20 * 3];
         
         prelude_list_for_each(list, tmp) {
                 entry = prelude_list_entry(tmp, regex_entry_t, list);
 
-                count = pcre_exec(entry->regex_compiled, entry->regex_extra,
-                                  str, len, 0, 0, ovector, 20 * 3);
-                if ( count <= 0 )
-                        continue;
+                if ( entry->rt->regex_regex ) {
+                        ret = pcre_exec(entry->rt->regex_regex, entry->rt->regex_regex_extra, str, len, 0, 0, NULL, 0);
+                        if ( ret < 0 )
+                                continue;
+                }
                 
-                prelude_log_debug(10, "[REGEX] string <%s> matched - count = %d\n", str, count);
+                prelude_log_debug(10, "[%s]: sending <%s>\n", entry->rt->plugin, str);
                 cb(entry->plugin, data);
         }
         
